#!/usr/bin/python3

# Skript	"parse-routes.py" 
# Author	Bernard Mai (contact: http://www.maiwolf.de/)
# Purpose	Convert route relations to ways and tag with map-specific refs.

# Version	18.8.2023

# The original script can be found here: https://github.com/darauble/Locus-Map-DIY


import os
import sys
import osmium

REL_TYPES = ["bicycle", "mtb", "hiking", "piste", "ski"]
HIKING_NETWORKS = ["rwn", "nwn", "iwn"]
CYCLING_NETWORKS = ["ucn", "lcn", "rcn", "ncn", "icn"]
TAGS = ["name", "route", "network", "ref", "piste:type"]
WINTER_TAGS = ["nordic", "skitour", "sled"]

all_ways = {}

class RelationsHandler(osmium.SimpleHandler):
	def __init__(self, all_ways):
		osmium.SimpleHandler.__init__(self)
		self.all_ways = all_ways

	def relation(self, r):
		if r.tags.get("route") in REL_TYPES and (r.tags.get("network") in HIKING_NETWORKS or r.tags.get("network") in CYCLING_NETWORKS or r.tags.get("piste:type") in WINTER_TAGS or r.tags.get("route") == "mtb"):
			relation = {
				"name": "",
				"route": "",
				"network": "",
			}

			#write only tags matching the tag list
			for k, v in r.tags:
				if k in TAGS:
					relation[k] = v

			#generate OpenOoutdoorMap specific tags
			#combine name and reg
			if "ref" in relation and relation["name"] != "":
				relation["name"] = relation["name"] + " (" + relation["ref"] + ")"
			#hiking routes
			if relation["route"] == "hiking" and "ref" in relation:
				relation["ref_hike"] = relation["ref"]
			if relation["route"] == "hiking":
				relation["hknetwork"] = relation["network"]
			#mtb routes
			if relation["route"] == "mtb" and "ref" in relation:
				relation["ref_mtb"] = relation["ref"]
			if relation["route"] == "mtb":
				relation["network"] = "mtb"
			#cycling routes
			if relation["route"] == "bicycle" and "ref" in relation:
				relation["ref_cycle"] = relation["ref"]

			#clean relation content
			if (relation["route"] == "hiking" or relation["route"] == "bicycle") and "ref" in relation:
				del relation["ref"]
			if relation["route"] == "hiking":
				del relation["network"]
			if relation["name"] == "":
				del relation["name"]
			if relation["route"] == "piste" or relation["route"] == "ski":
				del relation["network"]
			del relation["route"]

			#collect all ways of a relation
			for m in r.members:
				if m.type == "w":
					if m.ref not in self.all_ways:
						self.all_ways[m.ref] = []
					self.all_ways[m.ref].append(relation)

class WayHandler(osmium.SimpleHandler):
	def __init__(self, all_ways, way_writer):
		osmium.SimpleHandler.__init__(self)
		self.all_ways = all_ways
		self.way_writer = way_writer
		self.way_id = -70000000000

	def way(self, w):
		if w.id in self.all_ways:
			#self.way_writer.add_way(w)
			rel_count = len(self.all_ways[w.id])
			#for relation in self.all_ways[w.id]:

			for i in range(0, rel_count):
				relation = self.all_ways[w.id][i]
				way = w.replace(id=self.way_id, tags=relation)
				self.way_writer.add_way(way)
				self.way_id = self.way_id + 1


file_name = sys.argv[1]
output_name = sys.argv[2]

reader = osmium.io.Reader(file_name)
reader.close()

rel_parser = RelationsHandler(all_ways)
rel_parser.apply_file(file_name)

if os.path.exists(output_name):
	os.remove(output_name)

way_writer = osmium.SimpleWriter(output_name)
way_parser = WayHandler(all_ways, way_writer)
way_parser.apply_file(file_name)
way_writer.close()

#print("...parsing routes done")
